#include <iostream>
#include <string>
#include "Connect.h"

struct ProgOptions {
    bool dds;
    bool das;
    bool data;
    string url;
};

ProgOptions ParseArgs(int argc, char *argv[]);
void Usage(string argv0);

int main(int argc, char *argv[])
{
    ProgOptions opts = ParseArgs(argc, argv);

    string path = opts.url;
    string cexpr;
    if (path.contains("?")) {
	path = opts.url.before("?");
	cexpr = opts.url.after("?");
    }

    Connect *c = new Connect(path);
    if (c == 0) {
	cerr << "Could not open connection" << endl;
	return 1;
    }

    // Optionally get and print DAS
    if (opts.das) {
	if (!c->request_das()) {
	    cerr << "Could not get the DAS" << endl;
	    return false;
	}
	c->das().print();
    }

    // Get DDS: if requesting data, use request_data() otherwise request_dds()
    DDS *dds;
    if (opts.data) {
	dds = c->request_data(cexpr);
	if (!dds) {
	  cerr << "Could not get the DDS" << endl;
	  return 1;
	}
    } else if (opts.dds) {
	if (!c->request_dds()) {
	    cerr << "Could not get the DDS" << endl;
	    return 1;
	}
	dds = &(c->dds());
	dds->parse_constraint(cexpr);
    }

    if (opts.dds) {
	if (opts.data)
	    dds->print();
	else
	    dds->print_constrained();
    }
	    
    if (opts.data) {
        bool sequence_found = false;
	for (Pix p = dds->first_var(); p; dds->next_var(p)) {
	    BaseType *bt = dds->var(p);
	    if (bt->type_name() == "Sequence") { // special handling
	      ((Sequence*)(bt))->print_all_vals(cout, c->source(), dds, "", true);
	      sequence_found = true;
	    } else if (bt->type_name() == "Structure") {
	      ((Structure*)(bt))->print_all_vals(cout, c->source(), dds, "", true);
	    } else {
	      if(sequence_found)
		bt->deserialize(c->source(), dds);
	      bt->print_val(cout);
	    }
	}
    }

    delete c;
    return 0;
}

ProgOptions ParseArgs(int argc, char *argv[]) {
    ProgOptions o;

    if (argc < 2)
	Usage(argv[0]);
    o.dds = o.das = o.data = false;
    int i;
    for (i=1; i<argc-1; ++i) {
	if (strcmp(argv[i],"-dds") == 0)
	    o.dds = true;
	else if (strcmp(argv[i],"-das") == 0)
	    o.das = true;
	else if (strcmp(argv[i],"-data") == 0)
	    o.data = true;
	else
	    Usage(argv[0]);
    }
    o.url = argv[i];
    return o;
}

void Usage(string argv0) {
    cerr << "Usage: " << argv0 << " [-dds] [-das] [-data] url" << endl;
    cerr << "\twhere -dds indicates that the DDS should be printed" << endl;
    cerr << "\t      -das indicates that the DAS should be printed" << endl;
    cerr << "\t      -data indicates that the data values should be printed" 
	<< endl;
    exit(1);
}

